/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user;

import java.util.Optional;
import java.util.stream.Stream;

import org.eclipse.microprofile.config.inject.ConfigProperty;
import org.jboss.resteasy.reactive.common.util.RestMediaType;

import de.ozgcloud.common.logging.OzgCloudLogging;
import io.quarkus.hal.HalCollectionWrapper;
import io.quarkus.hal.HalEntityWrapper;
import jakarta.inject.Inject;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.MediaType;

@OzgCloudLogging
@Path(UserProfileResource.USERS_PATH)
public class UserProfileResource {
	static final String PARAM_SEARCH_BY = "searchBy";
	static final String PARAM_DELETED = "deleted";
	static final String PARAM_ORGANISATIONS_EINHEIT_ID = "organisationsEinheitId";

	static final int LIMIT = 10;

	static final String USERS_PATH = "/api/userProfiles"; // NOSONAR
	static final String USER_PROFILE_RESOURCE_PATH_TEMPLATE = USERS_PATH + "/{id}"; // NOSONAR

	@Inject
	UserService userService;

	@Inject
	UserProfileResourceAssembler resourceAssembler;

	// TODO parameter in den resourceAssembler verschieben
	// TODO wozu ist der eigentlich gut? - bitte aus Request übernehmen
	@ConfigProperty(name = "ozgcloud.user-manager.url")
	String userManagerUrl;

	@GET
	@Produces({ MediaType.APPLICATION_JSON, RestMediaType.APPLICATION_HAL_JSON })
	public HalCollectionWrapper findUsers(
			@QueryParam(value = PARAM_SEARCH_BY) String searchBy,
			@QueryParam(value = PARAM_DELETED) Optional<Boolean> deleted,
			@QueryParam(value = PARAM_ORGANISATIONS_EINHEIT_ID) Optional<String> organisationsEinheitId) {

		var query = FindUserProfilesQuery.builder()
				.searchBy(searchBy)
				.organisationsEinheitId(organisationsEinheitId.orElse(null))
				.limit(LIMIT)
				.build();

		Stream<User> users;
		if (deleted.isEmpty()) {
			users = userService.findUsers(query);
		} else if (deleted.orElse(false)) {
			users = userService.findInactiveUsers(query);
		} else {
			users = userService.findActiveUsers(query);
		}
		return resourceAssembler.toUserProfileResourceList(users.toList(), userManagerUrl);
	}

	@GET
	@Path("/{id}")
	@Produces({ MediaType.APPLICATION_JSON, RestMediaType.APPLICATION_HAL_JSON })
	public HalEntityWrapper findById(@PathParam(value = "id") String id) {
		return resourceAssembler.toUserProfileResource(userService.getById(id), userManagerUrl);
	}
}