package de.ozgcloud.user.settings;

import static org.assertj.core.api.Assertions.*;

import jakarta.inject.Inject;

import org.bson.types.ObjectId;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import de.ozgcloud.user.UserTestFactory;
import de.ozgcloud.user.common.MongoDbTestProfile;
import io.quarkus.test.junit.QuarkusTest;
import io.quarkus.test.junit.TestProfile;

@QuarkusTest
@TestProfile(MongoDbTestProfile.class)
class UserSettingsRepositoryITCase {

	@Inject
	UserSettingsRepository repository;

	@Nested
	class TestUpdateByUserId {

		@BeforeEach
		void beforeEach() {
			repository.deleteAll();
		}

		@Test
		void shouldUpdateUserSettings() {
			var user = UserTestFactory.create();
			repository.persist(user);

			var settings = repository.updateByUserId(
					user.getId().toString(),
					UserSettings.builder().notificationsSendFor(NotificationsSendFor.ALL).build());

			assertThat(settings)
					.isPresent()
					.map(UserSettings::getNotificationsSendFor)
					.hasValue(NotificationsSendFor.ALL);
		}

		@Test
		void shouldReturnEmptyIfUserNotExist() {
			var user = UserTestFactory.create();
			repository.persist(user);

			var settings = repository.updateByUserId(
					new ObjectId().toString(),
					UserSettings.builder().notificationsSendFor(NotificationsSendFor.ALL).build());

			assertThat(settings).isEmpty();
		}

		@Test
		void shouldReturnEmptyIfInvalidUserId() {
			var user = UserTestFactory.create();
			repository.persist(user);

			var settings = repository.updateByUserId(
					"wrong_id",
					UserSettings.builder().notificationsSendFor(NotificationsSendFor.ALL).build());

			assertThat(settings).isEmpty();
		}

		@Test
		void shouldReturnEmptyIfUserIdNull() {
			var user = UserTestFactory.create();
			repository.persist(user);

			var settings = repository.updateByUserId(
					null,
					UserSettings.builder().notificationsSendFor(NotificationsSendFor.ALL).build());

			assertThat(settings).isEmpty();
		}
	}

}